// ============================================================================
// Project:	Deductive Database
// Filename:	load.h
// Purpose:	Parse and load data.
// Last Change:	09.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	LOAD_INCLUDED
#define LOAD_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef	STRTAB_INCLUDED
#include "strtab.h"
#endif

#ifndef	PRED_INCLUDED
#include "pred.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// LOAD_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long LOAD_MAGIC = 0x4C4F440AL;	// 'LOD\n'

//-----------------------------------------------------------------------------
// LOAD_HASHSIZE: Size of hash table.
//-----------------------------------------------------------------------------

static const int LOAD_HASHSIZE = 251; // Should be prime number

//=============================================================================
// Forward declaration for private type (predicate data):
//=============================================================================

class load_pred_c;

//=============================================================================
// Class for Data LOader:
//=============================================================================

class load_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	load_c();

	// Destructor:
	~load_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// add: Define predicate.
	void add(pred_t predicate);

	// Load data:
	bool load(str_t filename);

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	public:
	// Integrity check:
	str_t	check() const;

	private:
	// Magic number (identifies objects of this class for debugging).
	long	Magic;		// Must be "LOAD_MAGIC".
#endif

#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	load_c(const load_c& obj);		// Not implemented
	load_c& operator=(const load_c& obj);	// Not implemented

//-----------------------------------------------------------------------------
// Auxiliary Methods:
//-----------------------------------------------------------------------------

	private:

	// hash: Compute hash function for predicate name.
	int hash(str_t name_start, str_t name_end);

	// find_pred: Find predicate data for given predicate name.
	pred_t find_pred(str_t name_start, str_t name_end);


//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// Hash table:
	pred_t HashTab[LOAD_HASHSIZE];

//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Include File:
//=============================================================================

#endif

